'''
@author: Marcos
'''
import sys, threading, math, time
from vision import ColorTracker, Color
from core.nao_vision import naoVision
from core.nao_motion import naoMotion 
from naoqi import ALProxy
from time import sleep
from behaviours import PickR, PickL, DropR, DropL
from random import randrange


def enum(*sequential, **named):
        enums = dict(zip(sequential, range(len(sequential))), **named)
        return type('Enum', (), enums)
    
def toDegrees(value):
    return value * 180 / math.pi

def toRads(value):
    return math.pi * value / 180


class Control(object):
    '''
    classdocs
    '''
    
    def __init__(self, ip, port, vision, environment, camera=False):
        '''
        Constructor
        '''
        self.ip = ip
        self.port = port
        self.camera = camera
        self.environment = environment
        self.cameraID = 0
        self.modes = enum('QUESTIONING', 'IDENTIFYING', 'EXECUTING', 'TERMINATING')
        self.cardinals = enum ('NORTH', 'WEST', 'SOUTH', 'EAST')
        self.mode = self.modes.QUESTIONING
        self.colors = []
        
        self.goal = 0
        self.currentGoal = 1;
        
        self.speech = ALProxy("ALTextToSpeech", self.ip, self.port)
        self.memory = ALProxy("ALMemory", self.ip, self.port)
        self.vision = None
        self.motion = naoMotion(self.ip, self.port, self.environment)
        self.app = None
        self.orientation = self.cardinals.NORTH
        self.oldOrientation = None
        self.hand = None
        
        self.replanning = False
        self.panic = False
        self.repositionCount = 0
        self.visionReposition = 0
        self.picturesTaken = 0
        self.actionCount = 0
        
        self.currentError = 0
        self.totalSensorMeasurements = 0
        self.totalError = 0
        self.sensorReposition = 0
        
        self.timer = 0.05
        self.errorTreshold = 5
        self.monitoring = False
        self.min = 0
        self.max = 0
        self.aX = 0
        self.aY = 0
        self.aXC = 1
        self.aYC = 0
        self.counter = 0
        self.geomCounter = 0
        self.timeMonitoring = 0
        
        self.over9000 = False
    
    def setVision(self, vision):
        self.vision = vision
        
    def setApp(self, app):
        self.app = app
    
    def reload(self):
        self.speech = ALProxy("ALTextToSpeech", self.ip, self.port)
        self.memory = ALProxy("ALMemory", self.ip, self.port)
        self.vision = naoVision(self.ip, self.port, self.cameraID, "vision"+str(randrange(100)))
        self.motion = naoMotion(self.ip, self.port, self.environment)
        self.app = None
        self.initValues()
        
    def confirmation(self, message):
        message = "Press front button to "+message+", press rear button to cancel and press mid button to listen this message again"
        self.speech.say(message)
        while True:
            if self.memory.getData("Device/SubDeviceList/Head/Touch/Front/Sensor/Value") > 0:
                self.speech.say("OK!")
                return True
            if self.memory.getData("Device/SubDeviceList/Head/Touch/Rear/Sensor/Value") > 0:
                self.speech.say("Nope!")
                return False
            if self.memory.getData("Device/SubDeviceList/Head/Touch/Middle/Sensor/Value") > 0:
                self.speech.say(message)

    def getGoals(self):
        return len(self.colors)-1

    def setContainerColor(self):
        boxIdentifier = Color.Color(59, 74)
        self.colors.append(boxIdentifier)
    
    def identifyItems(self):
        self.motion.loadPosture()
        self.motion.goToPosture("StandInit", 5)
                
        radius = 110
        
        self.setContainerColor()
        
        ct = ColorTracker.ColorTracker()
        
        message = "Press front button to identify items, rear button to start the execution and mid button to listen this message again"
        self.speech.say(message)
        while True:
            if self.memory.getData("Device/SubDeviceList/Head/Touch/Front/Sensor/Value") > 0:
                if self.mode == self.modes.IDENTIFYING:
                    image = self.vision.getFrame()
                    hsvImage = ct.toHSV(image)
                    photo = ct.meanColorCirumference(hsvImage, radius, 20)
                    values = ct.getRange(photo, 5)
                    print values
                    if self.confirmation("store item"):
                        color = Color.Color(values[0], values[1])
                        self.colors.append(color)
                        message = "%1d items identified so far, press front button store item, rear button to stop identifying and mid button to resize capture area" % (len(self.colors)-1)
                        self.speech.say(message)
                if self.mode == self.modes.QUESTIONING:
                    self.speech.say("Time to identify items")
                    sleep(1)
                    message = "%1d items identified so far, press front button store item, rear button to stop identifying and mid button to resize capture area" % (len(self.colors)-1)
                    self.speech.say(message)
                    self.mode = self.modes.IDENTIFYING
                    
            if self.memory.getData("Device/SubDeviceList/Head/Touch/Rear/Sensor/Value") > 0:
                if self.mode == self.modes.QUESTIONING:
                    if len(self.colors) > 1:
                        self.speech.say("Starting planification system")
                        self.mode = self.modes.EXECUTING
                        if not self.camera:
                            self.app.closeAllWindows()
                            self.app.exit()
                            self.app.quit()
                        self.storeData()
                        #self.reload() 
                        break;
                    else:
                        self.speech.say("I don't know any items yet")
                if self.mode == self.modes.IDENTIFYING:
                    message = "%1d items identified so far" % (len(self.colors)-1)
                    self.speech.say(message)
                    if self.confirmation("stop identifying"):
                        self.mode = self.modes.QUESTIONING
                        message = "Press front button to identify items, rear button to start the execution and mid button to listen this message again"
                        self.speech.say(message)
                
                        
            if self.memory.getData("Device/SubDeviceList/Head/Touch/Middle/Sensor/Value") > 0:
                if self.mode == self.modes.IDENTIFYING:
                    radius = (radius+5) % 120
                    self.vision.setCalibrationCircle(radius)
                if self.mode == self.modes.QUESTIONING:
                    self.speech.say(message)
                    
                    
    def loadBehaviour(self, side):
        if side == 0:
            print "Trying to pick item with left hand"
            self.hand = "LHand"
            PickL.main(self.ip, self.port)
        elif side == 1:
            print "Trying to pick item with left hand"
            self.hand = "RHand"
            PickR.main(self.ip, self.port)
        elif side == 2:
            DropL.main(self.ip, self.port)
        elif side == 3:
            DropR.main(self.ip, self.port)
        else:
            print "no behaviour to be loaded" 
        return
    
    
    def performAction(self, action):
        if self.panic:
            self.panic()
            return
        action = action.strip()
        result = False
        if "TURN" in action:
            orientation = action.split( "#")[3]
            result = self.turnWithReposition(orientation)
        elif "CATCH" in action:
            result = self.catch()
        elif "MOVE" in action:
            result = self.move()
        elif "DROP" in action:
            result = self.drop()
        elif "FINISH" in action:
            self.state = self.modes.TERMINATING
            self.over9000 = True
            return
        else:
            print "Invalid action: ", action
            
        self.actionCount += 1
            
        return result
            
    def turn(self, orientation):
        value = None
        if orientation == "NORTH":
            value = self.cardinals.NORTH
        elif orientation == "WEST":
            value = self.cardinals.WEST
        elif orientation == "SOUTH":
            value = self.cardinals.SOUTH
        elif orientation == "EAST":
            value = self.cardinals.EAST
        else:
            print "invalid orientation ", orientation
            return -1
        temp = value
        value = self.orientation - value
        
        if value == 0:
            print "already facing that direction"
            return 0
        
        if abs(value) == 2:
            self.motion.turnRight()
            self.measureError(90)
            self.correctError()
            self.trackSensors()
            self.motion.turnRight()
        elif value == 1 or value == -3:
            self.motion.turnRight()
        elif value == -1 or value == 3:
            self.motion.turnLeft()
        
        self.oldOrientation = self.orientation
        self.orientation = temp
        
        return True
        
        
    def initValues(self):
        self.motion.loadMotion()
        self.motion.loadPosture()
        self.motion.setStiffnesses("Body", 1, 1)
        if self.cameraID == 0:
            self.motion.headPitchDownwards([0.05, 0.10, 0.15])
        else:
            self.motion.headPitchDownwards([-0.05, -0.15, -0.25])
     
    def initTestValues(self):
        if self.motion._motionProxy is None:
            self.motion.loadMotion()
        if self.motion._postureProxy is None:
            self.motion.loadPosture()
        if self.vision is None: 
            self.vision = naoVision(self.ip, self.port, self.cameraID, "vision"+str(randrange(100)))        
        
        self.mode = self.modes.EXECUTING
        
        self.motion.goToPosture("StandInit", 1)
        self.motion.setStiffnesses("Body", 1, 1)

        if self.cameraID == 0:
            self.motion.headPitchDownwards([0.05, 0.10, 0.15])
        else:
            self.motion.headPitchDownwards([-0.05, -0.15, -0.25])
        """
        ask for orientation?
        FIXED ORIENTATION -- NORTH
        """
    
    def catch(self):
        self.changeCamera()
        colorRange = self.colors[self.currentGoal].getRange()
        area = 0
        while area !=6 and area !=7:
            area = self.moveWithCamera()
        behaviour = area-6
        self.loadBehaviour(0)#(behaviour) #Right hand unsupported
        #centroid = self.vision.getCentroid(self.vision.getPicture(), colorRange[0], colorRange[1])
        #if self.replanning and (centroid != None):
        #    print "Failed picking, trying to re-pick"
        #    self.motion.moveBackward()
        #    return False
        
        self.vision.unsubscribe()
        self.cameraID = 0
        self.vision = naoVision(self.ip, self.port, self.cameraID, "vision"+str(randrange(100)))
        self.motion.moveTo(-0.1,0,0.03)
        self.currentGoal = 0
        return True
        

    def moveWithCamera(self):
        colorRange = self.colors[self.currentGoal].getRange()
        #while area != 6 and area != 7:
        centroid = self.vision.getCentroid(self.vision.getPicture(), colorRange[0], colorRange[1])
        self.picturesTaken += 1
        area = self.motion.getBestMove(centroid, self.cameraID)
        if area == -1:
            self.changeCamera()
        error = 1;
        #sleep(2)
        print "Current area: "+str(area)
        #calculate error only if object is away
        while( error > 0 and area >= 1 and area != 6 and area != 7):
            centroid = self.vision.getCentroid(self.vision.getPicture(), colorRange[0], colorRange[1])
            error = self.motion.reposition(centroid)
            self.picturesTaken += 1
            self.repositionCount += 1
            self.visionReposition += 1
        return area
    
    def move(self):
        #self.trackSensors()
        self.motion.moveXSquare(1)
        self.oldOrientation = self.orientation
        #self.measureError()
        #self.correctError()
        
        #robot1
        multiplier = 2
        self.motion.turnLeft(rad=toRads(12 * multiplier))
        self.motion.moveTo(0, 0.08 * multiplier, 0) #Punto rojo es el malo --> 0.06 #0.16
        self.motion.turnLeft(rad=toRads(12 * multiplier))
        self.repositionCount += 3
        
        """
        #robot2
        self.motion.turnLeft(rad=toRads(10))
        self.motion.moveTo(0, 0.1, 0)
        self.motion.turnLeft(rad=toRads(10))
        self.repositionCount += 3
        """
        #self.motion.stepLeft()
        #self.motion.stepLeft()
        #sleep(2)
        return True
    
    def turnWithReposition(self, orientation):
        self.trackSensors()
        result = self.turn(orientation)
        self.measureError(90)
        self.correctError()
        #sleep(2)
        return result
    
    def record(self):
        x = abs(self.memory.getData("Device/SubDeviceList/InertialSensor/GyroscopeX/Sensor/Value"))  #rad.s-1
        #y = self.memory.getData("Device/SubDeviceList/InertialSensor/GyroscopeY/Sensor/Value")   #rad.s-1
        #xC = self.memory.getData("Device/SubDeviceList/InertialSensor/AngleX/Sensor/Value")     #rads
        #yC = self.memory.getData("Device/SubDeviceList/InertialSensor/AngleY/Sensor/Value")     #rads
        self.aX += x
        #self.aY += abs(y)
        #self.aXC += xC
        #self.aYC += (yC)
        self.counter += 1
        if self.monitoring:
            threading.Timer(self.timer, self.record).start()
    
    def correctError(self):
        if abs(self.currentError) > self.errorTreshold:
            self.motion.turnLeft(toRads(self.currentError))
            self.totalError += abs(self.currentError)
            self.currentError = 0
            self.sensorReposition += 1
            self.repositionCount += 1
    
    def trackSensors(self):
        self.monitoring = True
        self.timeMonitoring = time.time()
        thread = threading.Thread(target = self.record)
        thread.start()
    
    def measureError(self, degrees):
        self.monitoring = False
        self.timeMonitoring = time.time() - self.timeMonitoring 
        aX = toDegrees(self.aX / self.counter * self.timeMonitoring)
        #aXAM = toDegrees((self.aXC ** 1./self.counter) * self.timeMonitoring)
        #ay = toDegrees(self.aY / self.counter * self.timeMonitoring)
        #error = (aXC + aX * 1.25) / 2
        error = aX * 1.40 - degrees
        if (self.orientation - self.oldOrientation) < 0:
            error = -error
            
        #self.currentError += error if self.aXC > 0 else -error
        self.currentError += error
        self.totalSensorMeasurements += 1
        
        print "aX: ", aX, "error: ", error, " curent error: ", self.currentError          
        self.aX = 0
        self.aXC = 1
        self.counter = 0
        self.geomCounter = 0

            
    def turnWithVision(self, orientation):
        colorRange = self.colors[self.currentGoal].getRange()
        result = self.turn(orientation)
        area = 1
        error = -2
        #while area != 6 and area != 7:
        centroid = 0
        while( error > 0 and area >= 1):
            centroid = self.vision.getCentroid(self.vision.getPicture(), colorRange[0], colorRange[1])
            error = self.motion.reposition(centroid)
            self.repositionCount += 1
            self.visionReposition += 1
        return result
    
    def moveWithConfirmation(self):
        self.move()
        self.move()
        return True
    
    def drop(self):
        self.changeCamera()
        area = 0
        while area !=6 and area !=7:
            area = self.moveWithCamera()
        hand = 2
        if self.hand == "RHand":
            hand = 3
        self.loadBehaviour(hand)
        self.vision.unsubscribe()
        self.cameraID = 0
        self.vision = naoVision(self.ip, self.port, self.cameraID, "vision"+str(randrange(100)))
        self.motion.moveTo(-0.1,0,0.03)
        self.goal += 1
        self.currentGoal = self.goal
        return True
    
    def changeCamera(self):
        print "changing camera and restarting video"
        self.cameraID = 1
        if self.camera == False:
            self.vision.unsubscribe()
        self.vision = naoVision(self.ip, self.port, self.cameraID, "vision"+str(randrange(100)))
        print "Head up!"
        self.motion.headPitchDownwards([-0.10, -0.25, -0.40])
    
    def panicListener(self):
        while(True):
            sleep(500)
            if self.mode == self.modes.TERMINATING:
                break
            if self.mode == self.modes.EXECUTING and self.memory.getData("Device/SubDeviceList/Head/Touch/Front/Sensor/Value") > 0:
                self.mode = self.modes.TERMINATING
                self.panic = True
                break
    
    def storeData(self):
        colorFile = open("colors.config", "w")
        for color in self.colors:
            colorRange = color.getRange()
            colorFile.write(str(colorRange[0])+","+str(colorRange[1])+"\n")
        colorFile.close()
    
    def panic(self):
        self.motion.goToPosture("StandInit", 1)
        sys.exit(0)
        
    def dumpData(self):
        return self.actionCount, self.repositionCount