'''
@author: Marcos
'''
import sys, getopt, time, atexit, os.path
from control.Control import Control
from control.Interpreter import Interpreter
from time import sleep
from threading import Thread
from PyQt4.QtGui import QApplication
from core.nao_vision import naoVision
from vision import Color
from random import randrange


global control, ip, port, camera, executionTime, environment

current_time_millis = lambda: int(round(time.time() * 1000))

def widget():
    global control, ip, port, camera
    app = QApplication(sys.argv)
    vision = naoVision(ip, port, 0, "vision"+str(randrange(100)))
    vision.initWidget()
    vision.setCalibrationCircle(40)
    #vision._imgWidget.filter(59,74)
    vision.displayCamera()
    sleep(4)
    control.setVision(vision)
    control.setApp(app)
    app.exec_()

def test():
    global control, environment
    control = Control(ip, port, environment, camera)
    control.initTestValues()
    control.setContainerColor()
    color = Color.Color(168, 3)
    control.colors.append(color)
    interpreter = Interpreter()
    interpreter.executePelea()
    conn = interpreter.waitForPelea()
    goals = str(control.getGoals())
    print "Sending ", goals, " goal(s)"
    conn.send(goals)
    while control.over9000 == False:
        print "Listening now"
        action = interpreter.listen()
        print "Action: ", action
        result = control.performAction(action)
        if(result):
            conn.send("aye")
        else:
            conn.send("nay")
    conn.close()
    end()

def testPath():
    global control, environment
    control = Control(ip, port, environment, camera)
    control.initTestValues()
    control.setContainerColor()
    color = Color.Color(168, 3)
    control.colors.append(color)
    control.vision.unsubscribe()
    interpreter = Interpreter()
    interpreter.executePelea()
    conn = interpreter.waitForPelea()
    goals = "No"
    print "Sending ", goals, " goal(s)"
    conn.send(goals)
    while control.over9000 == False:
        print "Listening now"
        action = interpreter.listen()
        print "Action: ", action
        result = control.performAction(action)
        if(result):
            conn.send("aye")
        else:
            conn.send("nay")
    conn.close()
    end()

    
def identify():
    global control, environment
    sleep(5)
    control = Control(ip, port, environment, camera)
    control.identifyItems()
    end()
    
def run():
    global control,environment
    sleep(5)
    control = Control(ip, port, environment, camera)
    control.initTestValues()
    loadColors()
    interpreter = Interpreter()
    interpreter.executePelea()
    conn = interpreter.waitForPelea()
    goals = str(control.getGoals())
    print "Sending ", goals, " goal(s)"
    conn.send(goals)
    while True:
        print "Listening now"
        action = interpreter.listen()
        print "Action: ", action
        result = control.performAction(action)
        printValues()
        if(result):
            conn.send("aye")
        else:
            conn.send("nay")


def end():
    printValues()

def error():
    global control
    sleep(15)
    control.panicListener()
    printValues()
    control.panic()

def loadColors():
    global control
    colorFile = open("colors.config", "r")
    for line in colorFile:
        values = line.split(",")
        color = Color.Color(float(values[0]), float(values[1]))
        control.colors.append(color)
    colorFile.close()

def printValues():
    global executionTime, control
    time = current_time_millis() - executionTime
    print "Time: ", time, "  Actions: ", control.actionCount, "Pictures taken: ", control.picturesTaken, "Repositions: ", control.repositionCount,"\nVision repositions: ", control.visionReposition, "Sensor repositions: ", control.sensorReposition, "\nSensor measurements: ", control.totalSensorMeasurements, "total sensor error: ", control.totalError

atexit.register(printValues)

def main(argv):
    
    global ip, port, camera, executionTime, colors, environment
    
    try:
        opts, args = getopt.getopt(argv[1:],"e:h:a:p:c") 
    except getopt.GetoptError:
        print "Usage: %s -i input -o output" % argv[0]
        sys.exit(2)
    
    camera = False;
    ip = "127.0.0.1"
    port = 9559
    colors = False
    
    for opt, arg in opts:
        if opt == '-h':
            print 'Main.py -a <IPAddress> -o <port>'
            sys.exit()
        elif opt == '--video':
            camera = True;
        elif opt in ("-a", "--address"):
            ip = arg
        elif opt in ("-p", "--port"):
            port = int(arg)
        elif opt in ("-c", "--colors"):
            path = os.path.dirname(os.path.realpath(__file__)) + "/colors.config"
            colors = os.path.isfile(path)
        elif opt in ("-e"):
            environment = int(arg)
            
    
    executionTime = current_time_millis()
    
    try:
        t1 = Thread(target=identify)
        t5 = Thread(target=run)
        t3 = Thread(target=test)
        t4 = Thread(target=error)
        t6 = Thread(target=testPath)
        t2 = Thread(target=widget)
        if colors:
            t5.start()
        else:
            t1.start()  #required
            #t3.start()
            #t4.start()
            t2.start() #required
            #t6.start()
        
    except BaseException, err:
        print err
    
if __name__ == '__main__':
    main(sys.argv)
    

    