from core.nao_node import naoNode
from time import sleep
from motion.Positioning import Positioning


class naoMotion(naoNode):
    
    def __init__(self, ip, port, environment):
        
        naoNode.__init__(self, ip, port)
        
        self.factor = 3
        self._ip = ip
        self.environment = environment
        self._port = port
        self._motionProxy = None
        self._navigationProxy = None
        self._postureProxy = None
        if self.environment == 0:
            self.factor = 1
        if self.environment == 1:
            self.factor = 3
        self._positioning = Positioning(self)
        
    def loadMotion(self):
        self._motionProxy = self.connectProxy("ALMotion")
        
    def loadNavigation(self):
        self._navigationProxy = self.connectProxy("ALNavigation")
        
    def loadPosture(self):
        self._postureProxy = self.connectProxy("ALRobotPosture")
        
    def setStiffnesses(self, names, values, time = 1):
        self._motionProxy.setStiffnesses(names, values, time)
        
    def walkTo(self, x, y, z):
        self._motionProxy.walkTo(x, y ,z)
        
    def openHand(self, hand):
        self._motionProxy(hand)
        
    def closeHand(self, hand):
        self._motionProxy(hand)
        
    def moveTo(self, x, y, z):
        self._motionProxy.moveTo(x, y, z)
        
    def setWalkTargetVelocity(self, x, y, z, frequency):
        self._motionProxy.setWalkTargetVelocity(x, y, z, frequency)
        
    def stopMove(self):
        self._motionProxy.stopMove()
    
    def moveForward(self):
        self.moveTo(1, 0, 0)
    
    def moveBackward(self):
        self.moveTo(-1, 0, 0)
    
    def stepRight(self):
        self.moveTo(0, self.factor * -0.035, self.factor * -0.07)
    
    def stepLeft(self):
        self.moveTo(0, self.factor * 0.035, self.factor * -0.01)

    def stepForward(self):
        self.moveTo(self.factor * 0.025, 0.0, self.factor * -0.025)
        
    def stepBackward(self):
        self.moveTo(self.factor * -0.01, 0, 0)
    
    def turnRight(self, rad = 1.570796327): #1/4 circumference "1.42"
        self.moveTo(0, 0, -rad)
    
    def turnLeft(self, rad = 1.570796327): #1/4 circumference "1.46"
        self.moveTo(0, 0, rad)
        
    def angleInterpolation(self, names, angleLists, timeLists, isAbsolute=True):
        self._motionProxy.angleInterpolation(names, angleLists, timeLists, isAbsolute)
        
    def headYawRight(self):
        self.angleInterpolation("HeadYaw", [-0.2, -0.35, -0.5], [1,2,3])
    
    def headYawLeft(self):
        self.angleInterpolation("HeadYaw", [0.2, 0.35, 0.5], [1,2,3])
        
    def headPitchUpwards(self, joints = [-0.2, -0.35, -0.5]):
        self.angleInterpolation("HeadPitch", joints, [1,2,3])

    def headPitchDownwards(self, joints = [0.2, 0.35, 0.5]):
        self.angleInterpolation("HeadPitch", joints, [1,2,3])
        
    def getPostures(self):
        return self._postureProxy.getPostureList()
    
    def goToPosture(self, posture, velocity):
        return self._postureProxy.goToPosture(posture, velocity)
    
    def moveXSquare(self, times=1):
        if self.environment == 0:
            self.moveTo(0.55*times, 0.0, 0.0) # lab distance
        elif self.environment == 1:
            self.moveTo(0.45*times, 0.0, 0.0) # corridor distance

        
    def getAngles(self, names, useSensors):
        return self._motionProxy.getAngles(names, useSensors)

    def isHeadMoved(self):
        #Default pos = -0.17000000178813934
        position = self.getAngles("Head", False)
        if(float(position[1]) < 0.1 and float(position[1]) > -0.1):
            return False
        return True


    #for distance in squares <= 1
    def getBestMove(self, centroid, cameraID):
        proximity = None
        value = None
        if cameraID == 1:
            print ("I'm very near, attempt best approach")
            value = self._positioning.bestMove(centroid)
        else:
            proximity = self._positioning.getProximity(centroid)
            print "Proximity ",proximity
        if proximity != 1 and proximity != None:
            print ("step forward")
            self.moveXSquare(0.5)
            value = 10 if proximity == 0 else 0
            #sleep(1)
            position = self.getAngles("Head", False)
            if float(position[1]) < 0.25:
                self.headPitchDownwards(joints = [0.2, 0.25, 0.3])
            
        elif proximity == 1:
            value = -1
        return value
        
    def reposition(self, centroid): 
        distance = 30
        print "measuring error"
        error = self._positioning.isBetween(centroid, distance)
        self._positioning.reposition(error)
        #sleep(2)
        return error

        
        