#!/usr/bin/env python

#
# Python node to use Nao's vision capabilities
# This code is currently compatible to NaoQI version 1.12.3
#
# Copyright 2012 - 2014 Moises Martinez, University Carlos III
#            2014 Marcos Zimmermann Casado, University Carlos III
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    # Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#    # Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#    # Neither the name of the Unversity Carlos III nor the names of its
#       contributors may be used to endorse or promote products derived from
#       this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#


from core.nao_node import naoNode
from vision import ColorTracker, ImageWidget, Color
import vision_definitions
import datetime
import Image

class naoVision(naoNode):
    
    def __init__(self, ip, port, cameraID, name):
        
        naoNode.__init__(self, ip, port)
        
        self._ip = ip
        self._port = port
        self._cameraID = cameraID
        self._videoProxy = self.connectProxy("ALVideoDevice")
        self._imgClient = None
        self._name = name
        self._registerImageClient()
        self._colorTracker = ColorTracker.ColorTracker();
        self._imgWidget = None
        self._colors = []
        
    def initWidget(self):
        self._imgWidget = ImageWidget.ImageWidget(self._videoProxy, self._imgClient)

                
    def _registerImageClient(self):
        resolution = vision_definitions.kQVGA  # 320 * 240
        colorSpace = vision_definitions.kRGBColorSpace
        self._imgClient = self._videoProxy.subscribe(self._name, resolution, colorSpace, 5)

        # Select camera.
        self._videoProxy.setParam(vision_definitions.kCameraSelectID,
                                  self._cameraID)
        
    def changeCamera(self, camera):
        self._cameraID = camera
        self._videoProxy.setParam(vision_definitions.kCameraSelectID,
                                  self._cameraID)
        
    def displayCamera(self):
        self._imgWidget.show()
        
    #Returns PIL Image
    def getFrame(self):
        return self._imgWidget.getPILImage()
    
    #Returns Raw Image
    def getImage(self):
        return self._imgWidget.getImage()
    
    def getQImage(self):
        return self._imgWidget.getQImage()
    
    def getPicture(self):
        naoImage = self._videoProxy.getImageRemote(self._imgClient)
        PILImage = Image.fromstring("RGB", (naoImage[0], naoImage[1]), naoImage[6])
        return PILImage
    
    def getCentroid(self, image, minH, maxH):
        centroid = self._colorTracker.colorFilter(minH, maxH, image, color=True, onlyCentroid=True)
        return centroid
    
    def getImageWidth(self):
        return self._colorTracker._imgWidth
    
    def getImageHeight(self):
        return self._colorTracker._imgHeight
    
    def addColor(self, color):
        newColor = Color.Color(color)
        self._colors[newColor._id] = newColor
    
    def setCalibrationCircle(self, radius):
        self._imgWidget.showCalibrationCircle(radius)
    
    def findObject(self, color_id):
        self._colors[color_id]
    
    
    def findObjects(self, colors):
        '''
        for color in colors
            self.findObject(color)
        '''
    
    def getObjectPosition(self, color):
        return self._colorTracker.getPosition(self, color, self.getFrame())

    '''
    def __del__(self):
        self._videoProxy.unsubscribe(self._imgClient)
    '''
    def unsubscribe(self):
        self._videoProxy.unsubscribe(self._imgClient)
   
    def getFrameXML(self):

        message = ''
        image = self.getImage()

        if (image != None):
            message += '<?xml version="1.0" encoding="UTF-8" ?>'
            message += '<image/>'
            message += '<atom predicate=\'width\'>%s</atom>' % image[0];
            message += '<atom predicate=\'height\'>%s</atom>' % image[1];
            message += '<atom predicate=\'step\'>%s</atom>' % image[2] * image[0];
            message += '<atom predicate=\'header_stamp\'>%s</atom>' % image[5];
            message += '<atom predicate=\'data\'>%s</atom>' % image[6];
            message += '<atom predicate=\'bigendian\'>False</atom>';
            message += '<atom predicate=\'encoding\'>rgb8</atom>';
            message += '<atom predicate=\'error\'>0</atom>';
            message += '</image/>'
            print message

"""
    def publishImage(self):

        if (self.active):

            print('Sending image')

            image = self.cameraProxy.getImageRemote(self.camera_id)

            if (image != None):
                message = Image()
                message.width = image[0]
                message.height = image[1]
                message.step = image[2] * image[0]
                message.header.stamp.secs = image[5]
                message.data = image[6]
                message.is_bigendian = False
                message.encoding = 'rgb8'

                self.image_pub.publish(buffer)
"""
    

