from cv2 import cv
from collections import Counter
import math

class ColorTracker:

    

    def __init__(self, camera = None):
        #cv.NamedWindow( color_tracker_window, 1 )
        self._imgWidth = 320
        self._imgHeight = 240
        
        if(camera != None):
            #Camera recording
            self.capture = cv.CaptureFromCAM(0)

    
    def customInRanges(self, image, minRange, maxRange, tresholdedImage):
        
        joinSides = False
        reps = 1
        temp = None
        
        if minRange[0] > maxRange[0]:
            joinSides = True
            reps = 2    
            
        for k in range (0,reps):        
            if joinSides:
                if k == 0:
                    temp = maxRange[0]
                    maxRange[0] = 180
                if k == 1:
                    maxRange[0] = temp
                    minRange[0] = 0
                
            for i in range (0, self._imgHeight):
                for j in range (0, self._imgWidth):
                    pixel = cv.Get2D(image, i, j) #0 = hue, 1 = saturation, 2 = value
                    monoPixel = [0.0]
                        #hue comparison                                              saturation comparison                                        value comparison
                    if (pixel[0] >= minRange[0] and pixel[0] <= maxRange[0]) and (pixel[1] >= minRange[1] and pixel[1] <= maxRange[1]) and (pixel[2] >= minRange[2] and pixel[2] <= maxRange[2]):
                        monoPixel = [255.0]
                        cv.Set2D(tresholdedImage, i, j, monoPixel)
                    else:
                        if k == 0:
                            cv.Set2D(tresholdedImage, i, j, monoPixel)
            
        return tresholdedImage


    def findCentroidArea(self, img, thresholded_img, center=False):
        moments = cv.Moments(cv.GetMat(thresholded_img), 0)
        area = cv.GetCentralMoment(moments, 0, 0)

        x = None
        y = None
        mid = None
        #there can be noise in the video so ignore objects with small areas 
        if(area > 40000):
            #determine the x and y coordinates of the center of the object 
            #we are tracking by dividing the 1, 0 and 0, 1 moments by the area 
            x = int(cv.GetSpatialMoment(moments, 1, 0)/area)
            y = int(cv.GetSpatialMoment(moments, 0, 1)/area)
            
            #print 'x: ' + str(x) + ' y: ' + str(y) + ' area: ' + str(area) 

            #create an overlay to mark the center of the tracked object 
            overlay = cv.CreateImage(cv.GetSize(img), 8, 3)

            cv.Circle(overlay, (x, y), 2, (255, 255, 255), 5)
            
            #cv.Circle(overlay, (self._imgWidth/3, self._imgHeight), 2, (255, 255, 255), 5)
            #cv.Circle(overlay, (0, self._imgHeight/2), 2, (255, 255, 255), 5)
            #cv.Circle(overlay, (self._imgWidth/3, self._imgHeight/2), 2, (255, 255, 255), 5)
        
            cv.Add(img, overlay, img)

            mid = [str(x), str(y)]
        if center:
            newImg = cv.CreateImage(cv.GetSize(img), cv.IPL_DEPTH_8U, 3)
            cv.CvtColor(img, newImg, cv.CV_HSV2BGR)
            #cv.SaveImage("../../images/getPicture.png", newImg)
            return mid
        
        return img
    
    
    def findContours(self, img, thresholded_img):
        
        #cv.ShowImage("Threshold",thresholded_img)
        
        storage = cv.CreateMemStorage(0)
        # find contours in the threshold image
        contour = cv.FindContours(thresholded_img,storage, cv.CV_RETR_CCOMP, cv.CV_CHAIN_APPROX_SIMPLE)
        points = []
        
        cv.SaveImage("../../images/7-cv_img_contours.png", thresholded_img)
        
        biggestOnly = True
        maxArea = 0
        bound_rect = None
        count = 0
                
        while contour:
            # Draw bounding rectangles
            bound_rect = cv.BoundingRect(list(contour))

            if not biggestOnly:
                pt1 = (bound_rect[0], bound_rect[1])
                pt2 = (bound_rect[0] + bound_rect[2], bound_rect[1] + bound_rect[3])
                points.append(pt1)
                points.append(pt2)
                cv.Rectangle(img, pt1, pt2, cv.CV_RGB(255,0,0), 2)
                
            else:
                area = cv.ContourArea(contour)
                if area > maxArea:
                    maxArea = area
            
            #contour = contour.h_prev()
            if contour.h_next() is not None:
                contour = contour.h_next()
                count = count + 1
                
            else:
                break
        
        if biggestOnly and maxArea > 0:
            while count > 0 or maxArea == cv.ContourArea(contour):
                contour = contour.h_prev()
                count = count - 1
                
            pt1 = (bound_rect[0], bound_rect[1])
            pt2 = (bound_rect[0] + bound_rect[2], bound_rect[1] + bound_rect[3])
            points.append(pt1)
            points.append(pt2)
            cv.Rectangle(img, pt1, pt2, cv.CV_RGB(255,0,0), 2)
        
    
    
        #cv.Add(img,thresholded_img,thresh2)
        #cv.ShowImage("Real",thresh2)
        #cv.ShowImage("Contours",thresholded_img)
        

    
    
    def toHSV(self, image = None):
        if(image == None):
            img = cv.QueryFrame(self.capture)
        else:
            img = cv.CreateImageHeader(image.size, cv.IPL_DEPTH_8U, 3)
            cv.SetData(img, image.tostring())
            image.save("../../images/2-pre_cv_img.png", "PNG")

        cv.SaveImage("../../images/3-cv_img_original.png", img)

        #OpenCV color images have BGR channel order, image model was RGB
        cv.ConvertImage(img, img, cv.CV_CVTIMG_SWAP_RB)
        cv.SaveImage("../../images/4-cv_img_RedBlueSwap.png", img)

        #blur the source image to reduce color noise 
        cv.Smooth(img, img, cv.CV_BLUR, 3);
        cv.SaveImage("../../images/5-cv_img_blurred.png", img)

        #convert the image to hsv(Hue, Saturation, Value) for easier color tracking (hue)
        hsv_img = cv.CreateImage(cv.GetSize(img), 8, 3)
        cv.CvtColor(img, hsv_img, cv.CV_BGR2HSV)
        cv.SaveImage("../../images/6-cv_img_hsv.png", img)
        
        return hsv_img
    
    
    def colorFilter(self, minH, maxH, image, color=True, onlyCentroid=False):
        image.save("../../images/2-pre_cv_img.png", "PNG")

        img = cv.CreateImageHeader(image.size, cv.IPL_DEPTH_8U, 3)
        cv.SetData(img, image.tostring())
        cv.SaveImage("../../images/3-cv_img_original.png", img)

        #OpenCV color images have BGR channel order, image model was RGB
        cv.ConvertImage(img, img, cv.CV_CVTIMG_SWAP_RB)
        cv.SaveImage("../../images/4-cv_img_RedBlueSwap.png", img)
        #blur the source image to reduce color noise 
        cv.Smooth(img, img, cv.CV_BLUR, 3);
        cv.SaveImage("../../images/5-cv_img_blurred.png", img)

        #convert the image to hsv(Hue, Saturation, Value) for easier color tracking (hue)
        hsv_img = cv.CreateImage(cv.GetSize(img), 8, 3)
        cv.CvtColor(img, hsv_img, cv.CV_BGR2HSV)
        cv.SaveImage("../../images/6-cv_img_hsv.png", img)

        #limit all pixels that don't match the criteria. 
        #OpenCV uses 0-180 as a hue range for the HSV color model 
        thresholded_img =  cv.CreateImage(cv.GetSize(hsv_img), 8, 1)
        #cv.InRangeS(hsv_img, (minH, 100, 100), (maxH, 255, 255), thresholded_img)
        self.customInRanges(hsv_img, [minH, 50, 50], [maxH, 255, 255], thresholded_img)
        
        cv.SaveImage("../../images/7-cv_img_thresholded.png", thresholded_img)
        
        cv.Smooth(thresholded_img, thresholded_img, cv.CV_GAUSSIAN, 9, 9)
        cv.Erode(thresholded_img,thresholded_img,None,3)
        cv.Dilate(thresholded_img,thresholded_img,None,10)
        
        cv.SaveImage("../../images/7-cv_img_gaussian+eroded+dilated.png", thresholded_img)
        
        thresh2 = None
        thresh2 = cv.CreateImage (cv.GetSize (thresholded_img), 8, 1)
        
        
        #self.findContours(hsv_img, thresh2)
        
        imgArray = None
                
        if(color):
            newImg = cv.CreateImage(cv.GetSize(hsv_img), cv.IPL_DEPTH_8U, 3)
            if(onlyCentroid):
                centroid = self.findCentroidArea(hsv_img, thresholded_img, center=True)
                cv.CvtColor(self.findCentroidArea(hsv_img, thresholded_img), newImg, cv.CV_HSV2RGB)
                imgArray = newImg.tostring()
                overlay = cv.CreateImage(cv.GetSize(img), 8, 3)
                if centroid != None:
                    cv.Circle(overlay, (int(centroid[0]), int(centroid[1])), 2, (255, 255, 255), 20)
                    cv.Add(img, overlay, img)
                    cv.SaveImage("../../images/8-cv_img_areaFound.png", img)
                    
                return centroid
            cv.CvtColor(self.findCentroidArea(hsv_img, thresholded_img), newImg, cv.CV_HSV2RGB)
            imgArray = newImg.tostring()
            cv.SaveImage("../../images/8-cv_img_areaFound.png", img)
        else:
            newImg = cv.CreateImage(cv.GetSize(hsv_img), cv.IPL_DEPTH_8U, 3)
            cv.CvtColor(thresholded_img, newImg, cv.CV_GRAY2BGR)
            imgArray = newImg.tostring()
        
        return imgArray
    
    def getRange(self, collection, error):
        colors = list()
        for color, count in collection:
            print 'Hue %s: count %7d' % (color, count)
            if color < 181 and len(colors) == 0: #all colors but gray, white and black
                colors.append(color)
                
            elif color < 181 and (math.fabs(colors[0] - color) < 5): #colors near the predominant color
                colors.append(color)
        colors.sort()
        #print colors
        minH = (colors[0] - error + 180) % 181
        maxH = (colors[len(colors)-1] + error) % 181
        return [minH, maxH]
                 
    
    def _getColorValue(self, image, x, y):
        color  = cv.Get2D(image, y, x)                
        colorValue = color[0]
        
        if color[1] < 25:
            colorValue = 181 #Grey
            
        if color[2] < 25: 
            colorValue = 182 #Black
            
        if (color[2] > 230 and color[1] < 25):
            colorValue = 183 #White
            
        return colorValue 
    
    
    def meanColor(self, image, n):
        imageGrid = list()
        for x in range (0, 319):
            for y in range(0, 239):
                colorValue = self._getColorValue(image, x, y)
                imageGrid.append(colorValue)
                
        data = Counter(imageGrid)
        return data.most_common(n)
        
    def circumferencePoints(self, image, minX, maxX, y, grid):
        for x in range (minX, maxX+1):
            colorValue = self._getColorValue(image, x, y)
            grid.append(colorValue)
        
        return grid
        
    def meanColorCirumference(self, image, radius, n):
        imageGrid = list()
        
        x = radius
        y = 0
        radiusError = 1-x
        while x >= y:
            self.circumferencePoints(image, -x + self._imgWidth/2, x + self._imgWidth/2, y + self._imgHeight/2, imageGrid)
            self.circumferencePoints(image, -y + self._imgWidth/2, y + self._imgWidth/2, x + self._imgHeight/2, imageGrid)
            self.circumferencePoints(image, -x + self._imgWidth/2, x + self._imgWidth/2, -y + self._imgHeight/2, imageGrid)
            self.circumferencePoints(image, -y + self._imgWidth/2, y + self._imgWidth/2, -x + self._imgHeight/2, imageGrid)
            y = y+1
            if(radiusError<0):
                radiusError+=2*y+1
            else:     
                x = x-1
                radiusError+=2*(y-x+1)
        
        data = Counter(imageGrid)
        return data.most_common(n)
        
        
    #image -> RGB image
    def getPos(self, color, image = None):
        if(image == None):
            img = cv.QueryFrame(self.capture)
        else:
            img = cv.CreateImageHeader(image.size, cv.IPL_DEPTH_8U, 3)
            cv.SetData(img, image.tostring())
            image.save("../../images/2-pre_cv_img.png", "PNG")

        cv.SaveImage("../../images/3-cv_img_original.png", img)

        #OpenCV color images have BGR channel order, image model was RGB
        cv.ConvertImage(img, img, cv.CV_CVTIMG_SWAP_RB)
        cv.SaveImage("../../images/4-cv_img_RedBlueSwap.png", img)

        #blur the source image to reduce color noise 
        cv.Smooth(img, img, cv.CV_BLUR, 3);
        cv.SaveImage("../../images/5-cv_img_blurred.png", img)

        #convert the image to hsv(Hue, Saturation, Value) for easier color tracking (hue)
        hsv_img = cv.CreateImage(cv.GetSize(img), 8, 3)
        cv.CvtColor(img, hsv_img, cv.CV_BGR2HSV)
        cv.SaveImage("../../images/6-cv_img_hsv.png", img)

        
        #limit all pixels that don't match the criteria. 
        #OpenCV uses 0-180 as a hue range for the HSV color model 
        #Red: 165 - 10 (H)
        #Blue: 100 - 125 (H) 
        
        thresholded_img =  cv.CreateImage(cv.GetSize(hsv_img), 8, 1)
        cv.InRangeS(hsv_img, (100, 100, 100), (125, 255, 255), thresholded_img)
        #cv.InRangeS(hsv_img, (0, 100, 100), (20, 255, 255), thresholded_img)
        
        cv.SaveImage("../../images/7-cv_img_limitedPixels.png", thresholded_img)
        
        #determine the objects moments and check that the area is large  
        #enough to be the object 
        moments = cv.Moments(cv.GetMat(thresholded_img), 0)
        area = cv.GetCentralMoment(moments, 0, 0)

        x = None
        y = None
        #there can be noise in the video so ignore objects with small areas 
        if(area > 10000):
            #determine the x and y coordinates of the center of the object 
            #we are tracking by dividing the 1, 0 and 0, 1 moments by the area 
            x = int(cv.GetSpatialMoment(moments, 1, 0)/area)
            y = int(cv.GetSpatialMoment(moments, 0, 1)/area)
            
            print 'x: ' + str(x) + ' y: ' + str(y) + ' area: ' + str(area) 

            #create an overlay to mark the center of the tracked object 
            overlay = cv.CreateImage(cv.GetSize(img), 8, 3)

            cv.Circle(overlay, (x, y), 2, (255, 255, 255), 20)
            cv.Add(img, overlay, img)
            #add the thresholded image back to the img so we can see what was  
            #left after it was applied 
            
            cv.Merge(thresholded_img, None, None, None, img)
            cv.SaveImage("../../images/8-cv_img_areaFound.png", img)
        
        return [str(x), str(y)]
    
    